<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.cloudoos.com
 * @since      1.0.0
 *
 * @package    Distrimedia_Integration
 * @subpackage Distrimedia_Integration/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @package    Distrimedia_Integration
 * @subpackage Distrimedia_Integration/admin
 * @author     Cloudoos <support@cloudoos.com>
 */
class Distrimedia_Integration_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	private $bpost = false;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of this plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $plugin_name, $version, $client, $logger, $bpost ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

		//Get Distrimedia Connection
		$this->client = $client;

		$this->logger = $logger;
		$this->bpost  = $bpost;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Distrimedia_Integration_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Distrimedia_Integration_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/distrimedia-integration-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Distrimedia_Integration_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Distrimedia_Integration_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

//		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/distrimedia-integration-admin.js', array( 'jquery' ), $this->version, false );

	}

	/**
	 * Method to sync products if applicable to distrimedia
	 *
	 * @param $post_id
	 * @param $post
	 * @param $update
	 * @param bool $child
	 *
	 * @return bool
	 */
	public function sync_product( $post_id, $post, $update, $child = false ) {
		$product = wc_get_product( $post_id );

		$isVariable = false;
		if ( $product->is_type( 'variable' ) && ! $child ) {
			$product = new WC_Product_Variable( $post_id );
			$isVariable = true;
			foreach ( $product->get_children() as $productId ) {
				$this->sync_product( $productId, false, false, $child = true );
			}
		}

		/*----------New Code Added-------------*/
		if ( $product->is_virtual() || $product->is_downloadable() || ! $product->get_sku() ) {
			return false;
		}
		/*----------New Code Added-------------*/

		$sku = $product->get_sku();
		if (isDistrimediaEanFormat($sku)) {
			$eanCode = $sku;
		}else {
			if ($isVariable) {
				$eanCode = get_post_meta($product->get_id(), 'variable_distrimedia_ean');
				if (is_array($eanCode)) {
					if (isset($eanCode[0])) {
						$eanCode = $eanCode[0];
					}else {
						$eanCode = $sku;
					}
				}
			}else {
				$eanCode = get_post_meta($product->get_id(), '_distrimedia_ean', true);
			}
			if (!isDistrimediaEanFormat($eanCode)) {
				$eanCode = '';
			}
		}

		$productData = ['EAN' => $eanCode];
		if ($sku != $eanCode) {
			$productData['ExternalRef'] = $sku;
		}
		$productData = array_merge($productData, [
			'Description1' => $product->get_name(),
			'Weight'       => $product->get_weight(),
			'Height'       => $product->get_height(),
			'Width'        => $product->get_width(),
			'Length'       => $product->get_length()
		]);
		$params = [
			'CreateProducts' => [
				'Product' => $productData
			]
		];

		$this->client->call( 'CreateProducts', $params );

		$this->productRequestInventory($product);
	}

	/**
	 * Grab product URLs where required by distrimedia
	 */
	public function sync_product_url() {
		if ( isset( $_GET['sync-distrimeida'] ) ) {
			$this->sync_product( $_GET['sync-distrimeida'], null, null );
		}

		if ( isset( $_GET['distrimedia-pushback'] ) ) {
			$cronMode = get_option( 'wc_settings_tab_distrimedia_cron_mode' );
			if ($cronMode == 'off') {
				return;
			}
			$rawdata = file_get_contents( "php://input" );
			if ( $rawdata ) {
				$responseXml = simplexml_load_string( $rawdata );
				if ( isset( $responseXml->children()->Inventory ) ) {
					error_log( 'Inventory Change(Push): ' . $rawdata );
					//Order Status Change Stuff
					$this->logger->syncProducts( $responseXml->children() );
				}
			}
		}

		if ( isset( $_GET['distrimedia-crons'] ) ) {
			$this->distrimedia_daily_crons();
		}
	}

	public function getTrackingUrl( $response ) {

		if (  $trackingUrl = $this->logger->getTrackingUrl( $response ) ) {
			return '<p class="form-field">
                        <label>Tracking URL</label>
                    </p><p><textarea>' . is_string($trackingUrl) ? $trackingUrl : print_r($trackingUrl)  . '</textarea></p>';
		}

	}


	/**
	 * Method to run daily crons and connection to WP CRON
	 */
	public function distrimedia_daily_crons() {
		//Download BPost if enabled
		$this->bpost->download_bpost_local();

		//Disabled on Patrick's request
		//Update Order Status for all orders
//		$distri_orders = $this->logger->getDmOrders();
//
//		foreach ( $distri_orders as $order ) :
//			//Get Latest Order Status from Distrimedia
//			$params = [
//				'RequestOrderStatus' => [
//					'OrderID' => $order->dm_order_id
//				]
//			];
//
//			$orderStatus = $this->client->call( 'RequestOrderStatus', $params );
//			$this->logger->syncOrderStatus( $orderStatus, $order->order_id );
//
//		endforeach;


		//Inventory Update for all products
		$products = wc_get_products( array(
			'virtual' => false,
			'downloadable' => false,
		) );

		foreach ( $products as $product ) :
			if ( $product->is_virtual() || $product->is_downloadable() || ! $product->get_sku() ) {
				return false;
			}
			if (strtolower($product->get_shipping_class()) != 'distrimedia') {
				continue;
			}
			if ( $product->is_type( 'variable' ) ) {
				foreach ( $product->get_children() as $productId ) {
					$this->productRequestInventory( $productId );
				}
			}

			$this->sync_product( $product->get_id(), null, null );

		endforeach;
	}

	/**
	 * This method will call RequestInventory from Distrimedia
	 * @param $product
	 */
	public function productRequestInventory($product) {
		if(is_numeric($product)) {
			$product = wc_get_product( $product );
		}

		$params = [
			'RequestInventory' => [
				'Product'   => $product->get_sku(),
				'Inventory' => true
			]
		];

		$response = $this->client->call( 'RequestInventory', $params );

		//Create Log Entry
		$this->logger->syncProducts( $response, $product );
	}

	/**
	 * Admin panel tab
	 *
	 * @param $product_data_tabs
	 *
	 * @return mixed
	 */
	public function distrimedia_tab( $product_data_tabs ) {
		$product_data_tabs['distrimedia-inventory'] = array(
			'label'  => __( 'Distrimedia Inventory', 'distrimedia-integration' ),
			'target' => 'distrimedia_inventory',
		);

		return $product_data_tabs;
	}

	/**
	 * Basic Distrimedia content
	 */
	public function distrimedia_tab_content() {
		global $woocommerce, $post, $wp;
		$distri_inventory = $this->logger->getProductInventory( $post->ID );
		require_once plugin_dir_path( __FILE__ ) . 'partials/products-distrimedia-inventory-tab.php';
	}

	/**
	 * Method box in WC product page
	 */
	public function order_meta_box() {
		add_meta_box( 'distrimedia_details', __( 'Distrimedia', 'woocommerce' ), array(
			$this,
			'distrimedia_details_metabox'
		), 'shop_order', 'side', 'core' );
	}

	/**
	 * Distrimedia Meta box in WC Product
	 */
	public function distrimedia_details_metabox() {
		global $post;

		// Getting an instance of the order object
		$order        = wc_get_order( $post->ID );
		$distri_order = $this->logger->getDmOrder( $post->ID );
		$orderLogs    = $this->logger->getOrderLogs( $post->ID );

		require_once plugin_dir_path( __FILE__ ) . 'partials/order-distrimedia-metabox.php';
	}


	/**
	 * @param $settings_tabs
	 *
	 * @return mixed
	 */
	public function add_settings_tab( $settings_tabs ) {
		$settings_tabs['settings_tab_distrimedia'] = __( 'Distrimedia', 'distrimedia-integration' );

		return $settings_tabs;
	}

	public function settings_tab() {
		woocommerce_admin_fields( $this->get_settings() );
	}

	public function update_settings() {
		woocommerce_update_options( $this->get_settings() );

		//License Validation
		$this->licenseValidate();
	}

	/**
	 * Settings page for this plugin
	 * @return mixed|void
	 */
	public function get_settings() {
		$licenseStatus = woocommerce_settings_get_option( 'wc_settings_tab_distrimedia_valid_license' ) ? 'Activated' : 'Invalid';
		$payment_gateways_obj = new WC_Payment_Gateways();
		$paymentGateWays = $payment_gateways_obj->get_available_payment_gateways();
		$paymentSelect = array();
		foreach ($paymentGateWays as $paymentId => $gateway) {
			$paymentSelect[$paymentId] = $gateway->title;
		}

		$settings      = array(
			'section_title' => array(
				'name' => __( 'Basic Settings', 'distrimedia-integration' ),
				'type' => 'title',
				'desc' => '',
				'id'   => 'wc_settings_tab_distrimedia_section_title'
			),
			'environment'       => array(
				'name' => __( 'Select Environment', 'distrimedia-integration' ),
				'type' => 'select',
				'id'   => 'wc_settings_tab_distrimedia_environment',
				'options' => array(
					'http://devewms.distrimedia.be:8080'        => 'http://devewms.distrimedia.be:8080',
					'http://ewms.distrimedia.be:8080'    => 'http://ewms.distrimedia.be:8080',
				),
			),
			'webshop'       => array(
				'name' => __( 'Webshop ID', 'distrimedia-integration' ),
				'type' => 'text',
				'id'   => 'wc_settings_tab_distrimedia_id'
			),
			'soappass'      => array(
				'name' => __( 'SOAP Pass', 'distrimedia-integration' ),
				'type' => 'text',
				'id'   => 'wc_settings_tab_distrimedia_pass'
			),
			'site_indicator' => array(
				'name' => __( 'Site Indicator', 'distrimedia-integration' ),
				'type' => 'text',
				'id'   => 'wc_settings_tab_distrimedia_site_indicator'
			),
			'dm_id_prefix'  => array(
				'name' => __( 'Order Prefix', 'distrimedia-integration' ),
				'type' => 'text',
				'id'   => 'wc_settings_tab_distrimedia_prefix'
			),
			'dm_before_confirm'  => array(
				'name' => __( 'Before Payment Methods', 'distrimedia-integration' ),
				'type' => 'multiselect',
				'options'   =>  $paymentSelect,
				'id'   => 'wc_settings_tab_distrimedia_before_confirm'
			),
			'license_key'   => array(
				'name' => __( 'License Key (' . $licenseStatus . ')', 'distrimedia-integration' ),
				'type' => 'text',
				'id'   => 'wc_settings_tab_distrimedia_license'
			),
			'cron_mode'        => array(
				'name'    => __( 'Distrimedia Inventory Sync Activation', 'distrimedia-integration' ),
				'type' => 'select',
				'id'   => 'wc_settings_tab_distrimedia_cron_mode',
				'options' => array(
					'hourly'        => __( 'Hourly', 'distrimedia-integration' ),
					'twicedaily'    => __( 'Twice a day', 'distrimedia-integration' ),
					'daily'         => __( 'Daily', 'distrimedia-integration' ),
					'weekly'        => __( 'Weekly', 'distrimedia-integration' ),
					'off'           => __( 'Off', 'distrimedia-integration' ),
				),
			),
			'ditrimedia_retry_orders'        => array(
				'name'    => __( 'Distrimedia Retry Orders', 'distrimedia-integration' ),
				'type' => 'select',
				'id'   => 'wc_settings_tab_distrimedia_retry_orders',
				'options' => array(
					'hourly'        => __( 'Hourly', 'distrimedia-integration' ),
					'twicedaily'    => __( 'Twice a day', 'distrimedia-integration' ),
					'daily'         => __( 'Daily', 'distrimedia-integration' ),
					'weekly'        => __( 'Weekly', 'distrimedia-integration' ),
					'off'           => __( 'Off', 'distrimedia-integration' ),
				),
			),
			'bpost_point_background_color'        => array(
				'name'    => __( 'Bpost Point Background Color', 'distrimedia-integration' ),
				'type' => 'select',
				'id'   => 'wc_settings_tab_bpost_point_background_color',
				'options' => array(
					'black'           => __( 'Black', 'distrimedia-integration' ),
					'white'           => __( 'White', 'distrimedia-integration' ),
					'red'        => __( 'Red', 'distrimedia-integration' ),
					'orange'    => __( 'Orange', 'distrimedia-integration' ),
					'yellow'         => __( 'Yellow', 'distrimedia-integration' ),
					'green'        => __( 'Green', 'distrimedia-integration' ),
					'blue'           => __( 'Blue', 'distrimedia-integration' ),
					'violet'           => __( 'Violet', 'distrimedia-integration' ),
				),
			),
			'section_end'   => array(
				'type' => 'sectionend',
				'id'   => 'wc_settings_tab_distrimedia_section_end'
			),


			'bpost_title'      => array(
				'name' => __( 'bpost Integration', 'distrimedia-integration' ),
				'type' => 'title',
				'desc' => '',
				'id'   => 'wc_settings_tab_distrimedia_bpost_title'
			),
			'bpost_account'    => array(
				'name' => __( 'Account Id', 'distrimedia-integration' ),
				'type' => 'text',
				'id'   => 'wc_settings_tab_distrimedia_bpost_account'
			),
			'bpost_url'        => array(
				'name'    => __( 'bpost Locator URL', 'distrimedia-integration' ),
				'type'    => 'text',
				'default' => 'https://pudo.bpost.be/Locator',
				'id'      => 'wc_settings_tab_distrimedia_bpost_url'
			),
			'bpost_radius'     => array(
				'name'    => __( 'bpost Search Radius', 'distrimedia-integration' ),
				'type'    => 'text',
				'default' => '5',
				'id'      => 'wc_settings_tab_distrimedia_bpost_radius'
			),
			'bpost_max_results'     => array(
				'name'    => __( 'bpost Max Results', 'distrimedia-integration' ),
				'type'    => 'text',
				'default' => '10',
				'id'      => 'wc_settings_tab_distrimedia_bpost_results'
			),
			'bpost_map_zoom'     => array(
				'name'    => __( 'Map Zoom Level', 'distrimedia-integration' ),
				'type'    => 'text',
				'default' => '10',
				'id'      => 'wc_settings_tab_distrimedia_bpost_zoom'
			),
			'bpost_google_api' => array(
				'name'    => __( 'Google API Key', 'distrimedia-integration' ),
				'type'    => 'text',
				'default' => '',
				'id'      => 'wc_settings_tab_distrimedia_google_api'
			),
			'bpost_end'        => array(
				'type' => 'sectionend',
				'id'   => 'wc_settings_tab_distrimedia_bpost_end'
			)
		);

		return apply_filters( 'wc_settings_tab_distrimedia_settings', $settings );
	}

	/**
	 * Validate License with central License validator server
	 */
	private function licenseValidate() {
		$license_key = woocommerce_settings_get_option( 'wc_settings_tab_distrimedia_license' );
		// API query parameters
		$api_params = array(
			'slm_action'        => 'slm_activate',
			'secret_key'        => YOUR_SPECIAL_SECRET_KEY,
			'license_key'       => $license_key,
			'registered_domain' => $_SERVER['SERVER_NAME'],
			'item_reference'    => urlencode( YOUR_ITEM_REFERENCE ),
		);

		// Send query to the license manager server
		$query    = esc_url_raw( add_query_arg( $api_params, YOUR_LICENSE_SERVER_URL ) );
		$response = wp_remote_get( $query, array( 'timeout' => 20, 'sslverify' => false ) );

		// Check for error in the response
		if ( is_wp_error( $response ) ) {
			echo "Unexpected Error! The query returned with an error.";
		}

//		var_dump($response);//uncomment it if you want to look at the full response

		// License data.
		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		// TODO - Do something with it.
//		var_dump($license_data);//uncomment it to look at the data

		if ( $license_data->result == 'success' || ( isset( $license_data->error_code ) && $license_data->error_code == 40 ) ) {//Success was returned for the license activation

			//Uncomment the followng line to see the message that returned from the license server
//			echo '<br />The following message was returned from the server: '.$license_data->message;

			//Save the license key in the options table
			update_option( 'wc_settings_tab_distrimedia_valid_license', $license_key );
		} else {
			//Show error to the user. Probably entered incorrect license key.
			//Uncomment the followng line to see the message that returned from the license server
//			echo '<br />The following message was returned from the server: '.$license_data->message;
			update_option( 'wc_settings_tab_distrimedia_valid_license', false );
		}
	}

	public function wpo_wcpdf_show_custom_attribute( $document_type, $item, $order ){
		if( !empty($order) && $document_type == 'invoice' ) {
			$product = wc_get_product($item['product_id']);
			$attributes = $product->get_attributes();
	
			foreach( $attributes as $type => $attribute ) {
				$taxonomy = $attribute->get_taxonomy_object();
				if( in_array( $type, array( 'pa_country-of-origin', 'pa_hs-code', 'pa_composition' ) ) ) { // attribute type
					echo '<dl class="meta">';
					echo '<dt class="my-custom-attribute">'.$taxonomy->attribute_label.':</dt><dd">'.$product->get_attribute( $taxonomy->attribute_name ).'</dd>';
					echo '</dl>';
				}
			}
		}
	}
}

// This is the secret key for API authentication. You configured it in the settings menu of the license manager plugin.
define( 'YOUR_SPECIAL_SECRET_KEY', '5d933c6fa82526.61394220' ); //Rename this constant name so it is specific to your plugin or theme.

// This is the URL where API query request will be sent to. This should be the URL of the site where you have installed the main license manager plugin. Get this value from the integration help page.
define( 'YOUR_LICENSE_SERVER_URL', 'https://cloudoos.com' ); //Rename this constant name so it is specific to your plugin or theme.

// This is a value that will be recorded in the license manager data so you can identify licenses for this item/product.
define( 'YOUR_ITEM_REFERENCE', 'Distrimedia Integration' ); //Rename this constant name so it is specific to your plugin or theme.