<?php

/**
 * @link              https://cloudoos.com
 * @since             1.0.0
 * @package           Distrimedia_Integration
 *
 * @wordpress-plugin
 * Plugin Name:       Integration for Distrimedia
 * Plugin URI:        #
 * Description:       Custom plugin that enables WooCommerce to communicate with Distrimedia SOAP API
 * Version:           3.0.19
 * Author:            Cloudoos
 * Author URI:        https://cloudoos.com
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain:       distrimedia-integration
 * Domain Path:       /languages
 * Distrimedia Integration Plugin
 *
 * This is a custom plugin coded to create an integration between distrimedia shipping platform
 * and woocommerce.
 *
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'DISTRIMEDIA_INTEGRATION_VERSION', '1.0.0' );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-distrimedia-integration-activator.php
 */
function activate_distrimedia_integration() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-distrimedia-integration-activator.php';
	Distrimedia_Integration_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-distrimedia-integration-deactivator.php
 */
function deactivate_distrimedia_integration() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-distrimedia-integration-deactivator.php';
	Distrimedia_Integration_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_distrimedia_integration' );
register_deactivation_hook( __FILE__, 'deactivate_distrimedia_integration' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-distrimedia-integration.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_distrimedia_integration() {

	//Only run this plugin if woocommerce is available
	if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
		$plugin = new Distrimedia_Integration();
		$plugin->run();
	}
}

// Display Fields
add_action('woocommerce_product_options_sku', 'woocommerce_product_custom_fields');
add_action('woocommerce_process_product_meta', 'woocommerce_product_custom_fields_save');
add_action('woocommerce_variation_options', 'woocommerce_product_custom_fields_var', 10, 3);
add_action( 'woocommerce_save_product_variation', 'save_variation_settings_fields', 10, 2 );

function woocommerce_product_custom_fields() {
	echo '<div class="product_custom_field">';
	woocommerce_wp_text_input(
		array(
			'id' => '_distrimedia_ean',
			'placeholder' => 'EAN',
			'label' => __('EAN code', 'woocommerce'),
			'type' => 'number',
		)
	);
	echo '</div>';
}

function woocommerce_product_custom_fields_var($loop, $variation_data, $variation ){
	echo '<div class="product_custom_field">';
	woocommerce_wp_text_input(
		array(
			'id' => 'variable_distrimedia_ean'. $loop,
			'name' => 'variable_distrimedia_ean['. $loop. ']',
			'class' => 'short',
			'placeholder' => 'EAN',
			'label' => __('EAN code', 'woocommerce'),
			'value' => get_post_meta( $variation->ID, 'variable_distrimedia_ean', true ),
			'type' => 'number',
		)
	);
	echo '</div>';
}

function woocommerce_product_custom_fields_save($post_id) {
	if (isset($_POST['_distrimedia_ean'])) {
		update_post_meta($post_id, '_distrimedia_ean', trim(sanitize_text_field($_POST['_distrimedia_ean'])));
	}
}

function save_variation_settings_fields( $variation_id, $loop ) {
	if (isset($_POST['variable_distrimedia_ean'][$loop]) ) {
		$variableDistrimediaEan = trim(sanitize_text_field($_POST['variable_distrimedia_ean'][$loop]));
		if ($variableDistrimediaEan && !isDistrimediaEanFormat($variableDistrimediaEan)) {
			echo "<script>alert('EAN must be 13-number code');</script>";
			exit();
		}
		update_post_meta( $variation_id, 'variable_distrimedia_ean', $variableDistrimediaEan);
	}
}

add_action( 'pre_post_update', 'mp_sync_on_product_save', 10, 2);

function mp_sync_on_product_save( $post_id, $post_data ) {
	if ($post_data['post_type'] !== 'product') {
		return;
	}
	$error = null;
	$distrimediaEan = isset($_POST['_distrimedia_ean'])?trim(sanitize_text_field($_POST['_distrimedia_ean'])):'';
	if ($distrimediaEan) {
		if (!isDistrimediaEanFormat($distrimediaEan)) {
			$error = 'EAN must be 13-number code';
		}
	}else {
		$product_shipping_class = null;
		$shipping_classes = get_terms( array('taxonomy' => 'product_shipping_class', 'hide_empty' => false ) );
		foreach ($shipping_classes as $row) {
			if ($row->term_id === (int)sanitize_text_field($_POST['product_shipping_class'])) {
				$product_shipping_class = strtolower($row->name);
				break;
			}
		}
		if ($product_shipping_class == 'distrimedia') {
			if (!isDistrimediaEanFormat(trim(sanitize_text_field($_POST['_sku'])))) {
				$error = 'Please fill in the EAN-code. This is required by Distrimedia';
			}
		}
	}
	if (!$error) {
		if (isset($_POST['variable_distrimedia_ean']) && is_array($_POST['variable_distrimedia_ean'])) {
			foreach ($_POST['variable_distrimedia_ean'] as $variableDistrimediaEan) {
				if (!isDistrimediaEanFormat(trim(sanitize_text_field($variableDistrimediaEan)))) {
					$error = 'Variable EAN must be 13-number code';
					break;
				}
			}
		}
	}
	if ($error) {
		update_option('product_save_notifications', json_encode(array('error', $error)));
		header('Location: ' . get_edit_post_link($post_id, 'redirect'));
		exit();
	}
}

function isDistrimediaEanFormat($code) {
	return preg_match('/^[0-9]{13}$/', $code);
}

function product_save_notification() {
	$notifications = get_option('product_save_notifications');
	if (!empty($notifications)) {
		$notifications = json_decode($notifications);
		switch ($notifications[0]) {
			case 'error': # red
			case 'updated': # green
			case 'update-nag': # ?
				$class = $notifications[0];
				break;
			default:
				$class = 'error';
				break;
		}

		$is_dismissable = '';
		if (isset($notifications[2]) && $notifications[2] == true)
			$is_dismissable = 'is_dismissable';

		echo '<div class="'.$class.' notice '.$is_dismissable.'">';
		echo '<p>'.$notifications[1].'</p>';
		echo '</div>';
		update_option('product_save_notifications', false);
	}
}

add_action( 'admin_notices', 'product_save_notification' );

run_distrimedia_integration();
