<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://www.cloudoos.com
 * @since      1.0.0
 *
 * @package    Distrimedia_Integration
 * @subpackage Distrimedia_Integration/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Distrimedia_Integration
 * @subpackage Distrimedia_Integration/includes
 * @author     Cloudoos <support@cloudoos.com>
 */
class Distrimedia_Integration {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Distrimedia_Integration_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;


	protected $client;
	protected $bpost;
	protected $logger;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'DISTRIMEDIA_INTEGRATION_VERSION' ) ) {
			$this->version = DISTRIMEDIA_INTEGRATION_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'distrimedia-integration';

		$this->load_dependencies();
		$this->set_locale();
		$this->intialize_client();
		$this->intialize_bpost();
		$this->intialize_logger();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Distrimedia_Integration_Loader. Orchestrates the hooks of the plugin.
	 * - Distrimedia_Integration_i18n. Defines internationalization functionality.
	 * - Distrimedia_Integration_Admin. Defines all hooks for the admin area.
	 * - Distrimedia_Integration_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-distrimedia-integration-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-distrimedia-integration-i18n.php';

		/**
		 * The class responsible for keeping logs up for everything coming from distrimedia
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-distrimedia-integration-logger.php';

		/**
		 * The class responsible for communicating with distrimedia soap server
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-distrimedia-integration-client.php';

		/**
		 * The class responsible for communicating with Bpost
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-distrimedia-integration-bpost.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-distrimedia-integration-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-distrimedia-integration-public.php';

		$this->loader = new Distrimedia_Integration_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Distrimedia_Integration_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Distrimedia_Integration_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Distrimedia_Integration_Admin( $this->get_plugin_name(), $this->get_version(), $this->get_client(), $this->get_logger(), $this->get_bpost() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

		//WooCommerce Hooks
		//$this->loader->add_action( 'save_post_product', $plugin_admin, 'sync_product', 10, 3 );

		//Distrimedia Tab on Products Page
		$this->loader->add_action( 'woocommerce_product_data_tabs', $plugin_admin, 'distrimedia_tab');
		$this->loader->add_action( 'woocommerce_product_data_panels', $plugin_admin, 'distrimedia_tab_content');
		$this->loader->add_action( 'init', $plugin_admin, 'sync_product_url' );

		//Distrimedia Custom Tab
		$this->loader->add_action( 'add_meta_boxes', $plugin_admin, 'order_meta_box' );

		//Settings Tab
		$this->loader->add_action( 'woocommerce_settings_tabs_array', $plugin_admin, 'add_settings_tab', 50 );
		$this->loader->add_action( 'woocommerce_settings_tabs_settings_tab_distrimedia', $plugin_admin, 'settings_tab' );
		$this->loader->add_action( 'woocommerce_update_options_settings_tab_distrimedia', $plugin_admin, 'update_settings' );
		//Cron job in admin
		$this->loader->add_action( 'distrimedia_daily_crons', $plugin_admin, 'distrimedia_daily_crons');
	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Distrimedia_Integration_Public( $this->get_plugin_name(), $this->get_version(), $this->get_client(), $this->get_logger(), $this->get_bpost() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

		//WooCommerce Hooks
//		$this->loader->add_action( 'init', $plugin_public, 'order_payment_complete' );
		//$this->loader->add_action( 'woocommerce_thankyou', $plugin_public, 'wc_thankyou_hook' ); // removing thankyou hook to avoid duplicated calls
		$this->loader->add_action( 'woocommerce_payment_complete', $plugin_public, 'order_payment_complete' );
		//$this->loader->add_action( 'woocommerce_order_status_completed', $plugin_public, 'order_payment_complete' );

		$this->loader->add_action( 'woocommerce_order_status_changed', $plugin_public, 'order_status_changed', 10, 3 );
		$this->loader->add_action( 'woocommerce_update_order', $plugin_public, 'order_updated');

		//Manual Call to Distrimedia
		$this->loader->add_action( 'pre_get_posts', $plugin_public, 'manual_dm_call' );
		$this->loader->add_action( 'query_vars', $plugin_public, 'register_query_vars' );

		//Pushback URL
		$this->loader->add_action( 'init', $plugin_public, 'distrimedia_pushback', 11 );

		//Inject Cron Operations
		$this->loader->add_action( 'wp', $plugin_public, 'distrimedia_cron_jobs');

		//Taping into the shipping API
		$this->loader->add_action( 'woocommerce_shipping_init', $this, 'shipping_init' );
		$this->loader->add_action( 'woocommerce_shipping_methods', $plugin_public, 'shipping_method' );

		//Extra billing Fields
		$this->loader->add_action( 'woocommerce_after_order_notes', $plugin_public, 'extra_checkout_fields' );
		$this->loader->add_action( 'woocommerce_checkout_process', $plugin_public, 'extra_checkout_fields_validation' );
		$this->loader->add_action( 'woocommerce_checkout_update_order_meta', $plugin_public, 'extra_checkout_order_meta' );

		//Cron job for orders that createorder is not yet executed, but in processing status.
		$this->loader->add_action( 'distrimedia_retry_order_cron_jobs', $plugin_public, 'distrimedia_retry_order_cron_jobs');
	}

	public function shipping_init() {
		/**
		 * The class responsible for defining all shipping related extenions to WooCommerce
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-distrimedia-integration-home.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-distrimedia-integration-locker.php';
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-distrimedia-integration-pickup.php';
	}

	private function intialize_client() {
		$this->client = new Distrimedia_Integration_Client();
	}

	private function intialize_bpost() {
		$this->bpost = new Distrimedia_Integration_Bpost();
	}

	private function intialize_logger() {
		$this->logger = new Distrimedia_Integration_Logger();
	}

	public function get_client() {
		return $this->client;
	}

	public function get_bpost() {
		return $this->bpost;
	}

	public function get_logger() {
		return $this->logger;
	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Distrimedia_Integration_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
