<?php


class Distrimedia_Integration_Logger {

	private $logTable = 'dm_order_logs';
	private $orderTable = 'dm_orders';
	private $productsTable = 'dm_products';
	private $wpdb = null;
	private $statusCodes = [
		'info'    => 1,
		'ok'      => 2,
		'error'   => 3,
		'unknown' => 9
	];

	public function __construct() {
		global $wpdb;
		$this->wpdb          = $wpdb;
		$this->logTable      = $this->wpdb->base_prefix . $this->logTable;
		$this->orderTable    = $this->wpdb->base_prefix . $this->orderTable;
		$this->productsTable = $this->wpdb->base_prefix . $this->productsTable;
	}

	/**
	 * Sync Distrimedia and WC records
	 *
	 * @param $soapResults
	 * @param bool $product
	 *
	 * @return bool|false|int
	 */
	public function syncProducts( $soapResults, $product = false ) {
		if ( ! $product ) {

			$product = $this->getProductBySku( (string) $soapResults->Inventory->EAN );
			if ( ! $product ) {
				return false;
			}
		}
		if ( isset( $soapResults->Inventory ) ) {

            /*----------New Code Added-------------*/
            //skip if this product does not require shipping. 
            //update stock status and quantity
            if(!$product->is_virtual() && !$product->is_downloadable()){
                if((float) $soapResults->Inventory->Claimable > 0){
                    $product->set_stock_status('instock');
                    $out_of_stock_staus = 'instock';
                }else{
                    $product->set_stock_status('outofstock');
                    $out_of_stock_staus = 'outofstock';
                }
                
                // 1. Updating the stock quantity
                update_post_meta($product->get_id(), '_stock', (float) $soapResults->Inventory->Claimable);

                // 2. Updating the stock status
                update_post_meta( $product->get_id(), '_stock_status', wc_clean( $out_of_stock_staus ) );

                // 3. Updating post term relationship
                wp_set_post_terms( $product->get_id(), $out_of_stock_staus, 'product_visibility', true );

                // And finally
                wc_delete_product_transients( $product->get_id() ); // Clear/refresh the variation cache
            }
            /*----------New Code Added-------------*/            
            
			$data = [
				'product_id' => $product->get_id(),
				'ean'        => $product->get_sku(),
				'inventory'  => json_encode( $soapResults->Inventory )
			];

			$db_product_id = $this->wpdb->get_var(
				$this->wpdb->prepare(
					"SELECT product_id FROM " . $this->productsTable . "
                    WHERE product_id = %d LIMIT 1",
					$product->get_id()
				)
			);

			if ( $db_product_id > 0 ) {
				//Update case
				return $this->wpdb->update( $this->productsTable, $data, [ 'product_id' => $product->get_id() ] );
			}

			return $this->wpdb->insert( $this->productsTable, $data );
		}

		return false;
	}

	/**
	 * Record a log entry for Admin use
	 *
	 * @param $soapResults
	 * @param $order_id
	 * @param $localMethod
	 * @param bool $orderSync
	 *
	 * @return bool
	 */
	public function log( $soapResults, $order_id, $localMethod, $orderSync = false ) {

		$data = [
			'order_id'     => $order_id,
			'log_type'     => $this->getLogStatus( $soapResults ),
			'local_method' => $localMethod,
			'message'      => json_encode( $soapResults )
		];

		$this->wpdb->insert( $this->logTable, $data );

		if ( $orderSync && $this->getLogStatus( $soapResults ) == $this->statusCodes['ok'] ) {
			$this->syncOrder( $soapResults, $order_id );
		}

		return true;
	}

	/**`
	 * Sync individual order with distrimedia
	 *
	 * @param $soapResults
	 * @param $orderNum
	 *
	 * @return bool|false|int
	 */
	private function syncOrder( $soapResults, $orderNum ) {

		if ( $order_id = $this->getOrderId( $soapResults ) ) {
			$data = [
				'order_id'    => $orderNum,
				'dm_order_id' => $order_id
			];

			$db_order_id = $this->wpdb->get_var(
				$this->wpdb->prepare(
					"SELECT order_id FROM " . $this->orderTable . "
                    WHERE order_id = %d LIMIT 1",
					$orderNum
				)
			);

			if ( $db_order_id > 0 ) {
				//Update case
				return $this->wpdb->update( $this->orderTable, $data, [ 'order_id' => $orderNum ] );
			} else {
				return $this->wpdb->insert( $this->orderTable, $data );
			}
		}

		return false;
	}

	/**
	 * Change order status
	 *
	 * @param $response
	 * @param bool $db_order_id
	 *
	 * @return false|int
	 */
	public function syncOrderStatus( $response, $db_order_id = false ) {
		if ( $db_order_id ) {
			$data = [
				'order_id'    => $db_order_id,
			];

		} else {
			$data = [
				'order_id'    => $this->filterOrderPrefix((string) $response->OrderNumber),
				'dm_order_id' => (string) $response->OrderID
			];

			$db_order_id = $this->wpdb->get_var(
				$this->wpdb->prepare(
					"SELECT order_id FROM " . $this->orderTable . "
                WHERE order_id = %d LIMIT 1",
					$data['order_id']
				)
			);
		}

		$data['dm_status'] = json_encode( $response );
		$data['order_id'] = $this->filterOrderPrefix($data['order_id']);

		$this->wooUpdateOrderStatus($data['order_id'], $response);

		$logData = [
			'Status' => 'OK',
			'Reason' => 'Order Status Updated: ' . $response->OrderStatus
		];

		//Log Entry
		$this->log( $logData, $data['order_id'], 'syncOrderStatus' );

		if ( $db_order_id > 0 ) {
			//Update case
			return $this->wpdb->update( $this->orderTable, $data, [ 'order_id' => $data['order_id'] ] );
		} else {
			return $this->wpdb->insert( $this->orderTable, $data );
		}

	}

	/**
	 * Update WC order status
	 *
	 * @param $order
	 * @param $response
	 *
	 * @return bool
	 */
	private function wooUpdateOrderStatus($order, $response) {

		if($response && !isset($response->OrderStatus)) {
			return false;
		}

		$status = strtolower((string) $response->OrderStatus);

		if(is_numeric($order)) {
			$order = wc_get_order($order);
		}

		if(!$order) {
			return false;
		}

		if(in_array($status, ['psh', 'shp']) && !$order->has_status('completed')) {

			$note = "Your Order Has Been Shipped Successfully.";
			if($trackingUrl = $this->getTrackingUrl($response)) {
				$note .= ' Track your order from this URL: ' . $trackingUrl;
			}


			$note = __($note, 'distrimedia-integration');
			$order->add_order_note( $note, true );
			$order->update_status( 'completed' );
		}

		return false;
	}

	/**
	 * Get tracking URL from soap request
	 * @param $response
	 *
	 * @return bool|mixed
	 */
	public function getTrackingUrl($response) {
		$trackingUrl = false;
		$response = (array)$response;
		
		if(isset($response['TrackAndTraceURL']) && !empty($response['TrackAndTraceURL'])) {
			$trackingUrl = is_array($response['TrackAndTraceURL']) ? $response['TrackAndTraceURL'][0] : (array)$response['TrackAndTraceURL'];
			//Use this line because API response with blank object and value comes with array which increase complexity
			$trackingUrl = is_array($trackingUrl) ? $trackingUrl[0] : $trackingUrl;
		}

		if( isset($response['TrackIDs']) && isset($response['TrackIDs']->Package) && !$trackingUrl) {
			$response = (array)$response['TrackIDs']->Package;
			
			$trackingUrl = is_array($response['TrackAndTraceURL']) ? $response['TrackAndTraceURL'][0] : (array)$response['TrackAndTraceURL'];
			//Use this line because API response with blank object and value comes with array which increase complexity
			$trackingUrl = is_array($trackingUrl) ? $trackingUrl[0] : $trackingUrl;
		}
		

		return $trackingUrl;
	}

	/**
	 * Clean order ID by removing prefix
	 * @param $orderId
	 *
	 * @return string|string[]|null
	 */
	private function filterOrderPrefix($orderId) {
		if ( is_integer( $orderId ) ) {
			return $orderId;
		}

		return preg_replace("/[^0-9]/", "", $orderId );
	}

	/**
	 *
	 * @param $sku
	 *
	 * @return false|WC_Product|null
	 */
	private function getProductBySku( $sku ) {
		$product_id = $this->wpdb->get_var( $this->wpdb->prepare( "SELECT post_id FROM %1s WHERE meta_key='_sku' AND meta_value='%2s' LIMIT 1", $this->wpdb->postmeta, $sku ) );

		if ( $product_id ) {
			return wc_get_product( $product_id );
		}

		return null;
	}

	/**
	 * @param $soapResults
	 *
	 * @return bool|string
	 */
	private function getOrderId( $soapResults ) {
		if ( isset( $soapResults->OrderID ) && $soapResults->OrderID ) {
			return (string) $soapResults->OrderID;
		}

		return false;
	}

	/**
	 * @param $results
	 *
	 * @return mixed
	 */
	private function getLogStatus( $results ) {
		if ( isset( $results->Status ) && isset( $this->statusCodes[ strtolower( $results->Status ) ] ) ) {
			return $this->statusCodes[ strtolower( $results->Status ) ];
		}

		if ( isset( $results['Status'] ) && isset( $this->statusCodes[ strtolower( $results['Status'] ) ] ) ) {
			return $this->statusCodes[ strtolower( $results['Status'] ) ];
		}

		return $this->statusCodes['unknown'];
	}

	/**
	 * @param $productId
	 *
	 * @return array|object|void|null
	 */
	public function getProductInventory( $productId ) {
		return $this->wpdb->get_row( "SELECT * FROM " . $this->productsTable . " WHERE product_id = '$productId'", ARRAY_A );
	}

	/**
	 * @param $order_id
	 *
	 * @return array|object|void|null
	 */
	public function getDmOrder( $order_id ) {
		return $this->wpdb->get_row( "SELECT * FROM " . $this->orderTable . " WHERE order_id = '$order_id'", ARRAY_A );
	}

	/**
	 * @return array|object|null
	 */
	public function getDmOrders() {
		return $this->wpdb->get_results( "SELECT * FROM " . $this->orderTable );
	}

	/**
	 * @param $order_id
	 *
	 * @return array|object|null
	 */
	public function getOrderLogs( $order_id ) {
		return $this->wpdb->get_results( "SELECT * FROM " . $this->logTable . " WHERE order_id = '$order_id' order by created_at desc limit 5" );
	}


	// Added new methods returning WC orders ids from dm orders
	public function getOrdersIds( ) {
		return array_column($this->wpdb->get_results( "SELECT `order_id` FROM " . $this->orderTable . " order by created_at desc", ARRAY_A ), 'order_id');
	}

	public function getNotCompleteOrderIds() {
		$return = array();
		$pending_orders_ids = array_diff(wc_get_orders(array('return' => 'ids', 'status' => ['processing', 'completed'], 'date_created' => '<'. (time() - 60 * 15))), $this->getOrdersIds());
		foreach ($pending_orders_ids as $order_id) {
			$order_logs = $this->getOrderLogs($order_id);
			$exists = false;
			foreach ($order_logs as $log) {
				$message = $log->message?json_decode($log->message, true):[];
				if (isset($message['Reason']) && strpos($message['Reason'], 'already exists') !== false) {
					$exists = true;
					break;
				}
			}
			if (!$exists) {
				$return[] = $order_id;
			}
		}
		return $return;
	}
}