<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://www.cloudoos.com
 * @since      1.0.0
 *
 * @package    Distrimedia_Integration
 * @subpackage Distrimedia_Integration/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @package    Distrimedia_Integration
 * @subpackage Distrimedia_Integration/public
 * @author     Cloudoos <support@cloudoos.com>
 */
class Distrimedia_Integration_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $plugin_name The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string $version The current version of this plugin.
	 */
	private $version;

	private $client;

	private $logger;

	private $bpost = false;

	private $manualOrder = false;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @param string $plugin_name The name of the plugin.
	 * @param string $version The version of this plugin.
	 *
	 * @since    1.0.0
	 */
	public function __construct( $plugin_name, $version, $client, $logger, $bpost ) {

		$this->plugin_name = $plugin_name;
		$this->version     = $version;

		//Get Distrimedia Connection
		$this->client = $client;

		$this->logger = $logger;
		$this->bpost  = $bpost;
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Distrimedia_Integration_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Distrimedia_Integration_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/distrimedia-integration-public.css', array(), $this->version, 'all' );
		wp_enqueue_style( 'dashicons' );

	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Distrimedia_Integration_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Distrimedia_Integration_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

//		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/distrimedia-integration-public.js', array( 'jquery' ), $this->version, false );

		if ( is_checkout() ) {
			wp_enqueue_script( 'distrimedia-checkout', plugin_dir_url( __FILE__ ) . 'js/bpost-map.js', array( 'jquery' ), $this->version, false );
			$googleApi = $this->bpost->get_google_api();
			if ( $googleApi ) {
				wp_localize_script( 'distrimedia-checkout', 'distrimedia_data',
					array(
						'bpostPoints' => $this->bpost->get_bpost_points(),
						'bpostRadius' => get_option( 'wc_settings_tab_distrimedia_bpost_radius', 5 ),
						'bpostResult' => get_option( 'wc_settings_tab_distrimedia_bpost_results', 10 ),
						'bpostZoom'   => get_option( 'wc_settings_tab_distrimedia_bpost_zoom', 13 ),
						'bpostBackgroundColor' => get_option( 'wc_settings_tab_bpost_point_background_color', 'green'),
					)
				);
				wp_enqueue_script( 'markerclustererplus', "https://unpkg.com/@google/markerclustererplus@4.0.1/dist/markerclustererplus.min.js", array(), '', true );
				wp_enqueue_script( 'googlemaps', "https://maps.googleapis.com/maps/api/js?&key=$googleApi&libraries=places&callback=initBpostMap", array(), '', true );
			}
		}
	}

	private function has_distrimeida_shipping( $order ) {
		if ( ! $order ) {
			return false;
		}

		if ( is_numeric( $order ) ) {
			$order = wc_get_order( $order );
		}

		$systemMethods = $this->shipping_method( array() );

		foreach ( $order->get_shipping_methods() as $method ) {
			if ( isset( $systemMethods[ $method->get_method_id() ] ) ) {
				return $method->get_method_id();
			}
		}

		return false;
	}

	/**
	 * Hook fired right after order is placed. Used this to process order before payment
	 *
	 * @param $order_id
	 */
	public function wc_thankyou_hook( $order_id ) {
		$order           = wc_get_order( $order_id );
		$paymentGateways = get_option( 'wc_settings_tab_distrimedia_before_confirm', array() );
		if ( in_array( $order->get_payment_method(), $paymentGateways ) ) {
			$this->order_payment_complete( $order );
		}
	}

	/**
	 * This callback will be called on payment complete
	 *
	 * @param $order_id
	 *
	 * @return bool|void
	 */
	public function order_payment_complete( $order_id ) {
//		$order_id = 990;
		if ( ! $order_id ) {
			return;
		}

		// Getting an instance of the order object
		if ( is_numeric( $order_id ) ) {
			$order = wc_get_order( $order_id );
		} else {
			$order = $order_id;
		}

		$order          = wc_get_order( $order_id );
		$shippingMethod = $this->has_distrimeida_shipping( $order );

		if ( ! $shippingMethod ) {
			$shippingMethod = 'distrimedia_home';
		}

		if ( ! $order || ! $shippingMethod ) {
			error_log( 'Order Failed: ' . $order_id );
			error_log( $order_id . ': This order not have suitable shipping method' );
			$shippingMethods = [];
			foreach ( $order->get_shipping_methods() as $method ) {
				$shippingMethods = $method->get_method_id();
			}
			error_log( print_r( $shippingMethods, true ) );

			return false;
		}

		$orderLines = [];
		foreach ( $order->get_items() as $key => $orderItem ) {
			$itemData = $orderItem->get_data();

			$isVariable = false;
			if ( isset( $itemData['variation_id'] ) && $itemData['variation_id'] > 0 ) {
				$product_id_to_get = $itemData['variation_id'];
				$isVariable = true;
			} else {
				$product_id_to_get = $itemData['product_id'];
			}
			
			if( $itemData['product_id']==0 ) continue;

			$product = wc_get_product( $product_id_to_get );
			
            $shipping_class = strtolower($product->get_shipping_class());
			if ($shipping_class != 'distrimedia') {
				continue;
			}

			//added the condition to check if a product is a downloadable or virtual product, hence shipping not required
			if ( $product->is_virtual() || $product->is_downloadable() ) {
				continue;
			}

			$sku = $product->get_sku();
			if (isDistrimediaEanFormat($sku)) {
				$eanCode = $sku;
			}else {
				if ($isVariable) {
					$eanCode = get_post_meta($product->get_id(), 'variable_distrimedia_ean');
					if (is_array($eanCode)) {
						if (isset($eanCode[0])) {
							$eanCode = $eanCode[0];
						}else {
							$eanCode = $sku;
						}
					}
				}else {
					$eanCode = get_post_meta($product->get_id(), '_distrimedia_ean', true);
				}
				if (!isDistrimediaEanFormat($eanCode)) {
					$eanCode = '';
				}
			}
			$productData = ['EAN' => $eanCode];
			if ($sku != $eanCode) {
				$productData['ExternalRef'] = $sku;
			}
			$productData = array_merge($productData, [
				'Description1' => $product->get_name(),
				'Weight'       => $product->get_weight(),
				'Height'       => $product->get_height(),
				'Width'        => $product->get_width(),
				'Length'       => $product->get_length()
			]);
			$orderLines[ 'OrderLine-dup-' . $key ] = [
				'ProductID' => $eanCode,
				'Pieces'    => $itemData['quantity'],
				'Product'   => $productData
			];
		}

		//If we don't have any physical product
		if ( empty( $orderLines ) ) {
			return true;
		}

		$language = defined( 'ICL_LANGUAGE_CODE' ) ? ICL_LANGUAGE_CODE : 'en';

//		$documentCount = $shippingMethod == 'distrimedia_home' ? 3 : 5;
		$documentCount = strpos( $shippingMethod, 'distrimedia_home' ) !== false ? 3 : 5;

		$orderParams = [
			'OrderNumber'         => $order->get_id(), // instead of $this->getOrderNumber( $order ) which means we always send order id
			'Reference'         =>   $order->get_order_number(),
//			'Customer'            => $customer,
			'Language'            => $language,
			'Currency'            => $order->get_currency(),
			'AdditionalDocuments' => $this->additionalDocuments( $order, $documentCount )
		];

		if ( strpos( $shippingMethod, 'distrimedia_home' ) === false ) {
			$bpost                   = $this->bpost->get_bpost_station( $order->get_meta( 'bpost_point' ) );
			$orderParams['Customer'] = $this->getCustomerObj( $order, $bpost );
//			$orderParams['ShipmentMethod'] = $this->bpost->has_shipping_method($shippingMethod, 'distrimedia_locker') ? 'BP247' : 'BPPUGO';
//			$orderParams['ServicePoint ID'] = $bpost->Id;
			$orderParams['Carrier'] = $this->bpost->has_shipping_method( $shippingMethod, 'distrimedia_locker' ) ? 'BP247' : 'BPPUGO';
		} else {
			$orderParams['Customer'] = $this->getCustomerObj( $order );
		}

		$params = [
			'Order' => $orderParams + $orderLines
		];

		$response = $this->client->call( 'CreateOrder', $params );

		//Create Log Entry
		$this->logger->log( $response, $order->get_id(), 'order_success', true );

		return true;
	}

	/**
	 * If we have any additional documents in request
	 *
	 * @param $order
	 *
	 * @return array|bool
	 */
	private function additionalDocuments( $order, $number = 3 ) {

		if ( ! function_exists( 'wcpdf_get_document' ) ) {
			return false;
		}

		//Return empty if country is EU
		if ( $this->is_eu( $order->get_billing_country() ) ) {
			return false;
		}

		$documents = [];

		for ( $i = 0; $i < $number; $i ++ ) {
			$documents[] = $order->get_id();
		}

		return [
			'AdditionalDocument' => [
				'FileTag' => 'INVOICE',
				'BinData' => base64_encode( wcpdf_get_invoice( $documents, true )->get_pdf() )
			]
		];
	}

	/**
	 * Method will help to know if country is in EU
	 *
	 * @param $isoCode
	 *
	 * @return bool
	 */
	private function is_eu( $isoCode ) {
		$countries       = [];
		$countries['AT'] = [ 'name' => 'Austria', ];
		$countries['BE'] = [ 'name' => 'Belgium', ];
		$countries['BG'] = [ 'name' => 'Bulgaria', ];
		$countries['CY'] = [ 'name' => 'Cyprus', ];
		$countries['CZ'] = [ 'name' => 'Czech Republic', ];
		$countries['DE'] = [ 'name' => 'Germany', ];
		$countries['DK'] = [ 'name' => 'Denmark', ];
		$countries['EE'] = [ 'name' => 'Estonia', ];
		$countries['ES'] = [ 'name' => 'Spain', ];
		$countries['FI'] = [ 'name' => 'Finland', ];
		$countries['FR'] = [ 'name' => 'France', ];
		$countries['GB'] = [ 'name' => 'United Kingdom', ];
		$countries['GR'] = [ 'name' => 'Greece', ];
		$countries['HU'] = [ 'name' => 'Hungary', ];
		$countries['HR'] = [ 'name' => 'Croatia', ];
		$countries['IE'] = [ 'name' => 'Ireland, Republic of (EIRE)', ];
		$countries['IT'] = [ 'name' => 'Italy', ];
		$countries['LT'] = [ 'name' => 'Lithuania', ];
		$countries['LU'] = [ 'name' => 'Luxembourg', ];
		$countries['LV'] = [ 'name' => 'Latvia', ];
		$countries['MT'] = [ 'name' => 'Malta', ];
		$countries['NL'] = [ 'name' => 'Netherlands', ];
		$countries['PL'] = [ 'name' => 'Poland', ];
		$countries['PT'] = [ 'name' => 'Portugal', ];
		$countries['RO'] = [ 'name' => 'Romania', ];
		$countries['SE'] = [ 'name' => 'Sweden', ];
		$countries['SI'] = [ 'name' => 'Slovenia', ];
		$countries['SK'] = [ 'name' => 'Slovakia', ];

		if ( isset( $countries[ $isoCode ] ) ) {
			return true;
		}

		return false;
	}

	/**
	 * This central method will control the order status change whenever required
	 *
	 * @param $order_id
	 * @param $this_status_transition_from
	 * @param $this_status_transition_to
	 *
	 * @return bool
	 */
	public function order_status_changed( $order_id, $this_status_transition_from, $this_status_transition_to ) {

		// Getting an instance of the order object
		$order  = wc_get_order( $order_id );
		$status = 'Startorder';

		if ( in_array( $this_status_transition_to, array( 'cancelled', 'refunded', 'failed' ) ) ) {
			$status = 'Cancel';
		} else {
			return true;
		}

		if ( ! $DmOrderId = $this->getDmOrderId( $order_id ) ) {
			return true;
		}

		$params = [
			'ChangeOrderStatus' => [
				'OrderID' => $DmOrderId,
				'Status'  => $status,
			]
		];

		$this->client->call( 'ChangeOrderStatus', $params );
	}

	/**
	 * For any update ot the WC Order
	 *
	 * @param $order_id
	 *
	 * @return bool
	 */
	public function order_updated( $order_id ) {
		if ( ! $order_id ) {
			return true;
		}


		if ( ! $DmOrderId = $this->getDmOrderId( $order_id ) ) {
			return true;
		}

		$order = wc_get_order( $order_id );
		$bpost = $order->get_meta( 'bpost_point' , false);
		if($bpost) {
			$bpost = $this->bpost->get_bpost_station( $order->get_meta( 'bpost_point' ) );
		}


		$params = [
			'ChangeCustomer' => [
				'OrderID'  => $DmOrderId,
				'Customer' => $this->getCustomerObj( $order_id, $bpost ),
			]
		];


		$this->client->call( 'ChangeCustomer', $params );
	}

	/**
	 * Grab Distrimedia Order number
	 *
	 * @param $order_id
	 *
	 * @return bool
	 */
	private function getDmOrderId( $order_id ) {
		$dm_order = $this->logger->getDmOrder( $order_id );

		if ( ! $dm_order || ! isset( $dm_order['dm_order_id'] ) ) {
			return false;
		}

		return $dm_order['dm_order_id'];
	}

	/**
	 * Registering Cron job
	 */
	public function distrimedia_cron_jobs() {
		foreach (['wc_settings_tab_distrimedia_cron_mode' => 'distrimedia_daily_crons', 'wc_settings_tab_distrimedia_retry_orders' => 'distrimedia_retry_order_cron_jobs'] as $optionId => $hookId) {
			$cronMode = get_option($optionId);
			if ($cronMode == 'off') {
				if (wp_next_scheduled($hookId)) {
					wp_clear_scheduled_hook($hookId);
				}
			} else {
				$event = wp_get_scheduled_event($hookId);
				if (!$event || !$event->timestamp || $event->schedule != $cronMode) {
					wp_schedule_event($event ? $event->timestamp : time(), $cronMode, $hookId);
				}
			}
		}
	}

	// Method for retrying uncompleted orders
	public function distrimedia_retry_order_cron_jobs() {
		// Get all orders that weren't completed by DM
		$orders_ids = $this->logger->getNotCompleteOrderIds();
		foreach ($orders_ids as $order_id) {
			$this->order_payment_complete( $order_id );
		}
	}

	/**
	 * Method for receiving pushback messages from distrimedia
	 */
	public function distrimedia_pushback() {
		if ( isset( $_GET['distrimedia-pushback'] ) ) {
			$rawdata = file_get_contents( "php://input" );
			error_log( 'Push Message: ' . $rawdata );
			if ( $rawdata ) {
				$responseXml = simplexml_load_string( $rawdata );

//				if(isset($responseXml->children('soap', true)->children()->OrderStatusChange)) {
				if ( isset( $responseXml->children()->OrderID ) ) {
					//Order Status Change Stuff
					$this->logger->syncOrderStatus( $responseXml->children() );
				}
			}
		}
	}

	/**
	 *
	 * Aquire Customer Object from WC in organized fashion
	 *
	 * @param $order
	 *
	 * @return array
	 */
	private function getCustomerObj( $order, $bpost = false ) {
		if ( is_numeric( $order ) ) {
			$order = wc_get_order( $order );
		}

		if ( $bpost ) {
			$customer = [
				'Name'        => str_replace( array( "&gt;", "&lt;", "&quot;", "&amp;", '&' ), array(
					">",
					"<",
					"\"",
					"and",
					"and"
				), $bpost->Name ),
				'Name2'       => $order->get_billing_first_name() . ' ' . $order->get_billing_last_name(),
				'Address1'    => $bpost->Street . ' ' . $bpost->Number,
				'PostalCode1' => $bpost->Zip,
				'City'        => $bpost->City,
				'Country'     => 'BE'
			];
		} else {
			$customer = [
				'Name'        => $order->get_shipping_first_name() . ' ' . $order->get_shipping_last_name(),
				'Name2'       => $order->get_shipping_company(),
				'Address1'    => $order->get_shipping_address_1(),
				'Address2'    => $order->get_shipping_address_2(),
				'PostalCode1' => $order->get_shipping_postcode(),
				'City'        => $order->get_shipping_city(),
				'Country'     => $order->get_shipping_country(),
			];
		}

		$customer['Telephone'] = $order->get_billing_phone();
		$customer['eMail']     = $order->get_billing_email();

		if ( $bpost ) {
			$customer['ServicePoint'] = $bpost->Id;
		}

		return $customer;
	}

	/**
	 * Capture wc order number
	 *
	 * @param $order
	 *
	 * @return string
	 */
	private function getOrderNumber( $order ) {
		if ( is_integer( $order ) ) {
			$order = wc_get_order( $order );
		}

		$prefix = get_option( 'wc_settings_tab_distrimedeia_prefix', true );

		return $prefix . $order->get_order_number();
	}

	public function manual_dm_call($query) {
		global $wp_query;
		if (empty($wp_query)) {
			$wp_query = $query;
		}
		if ( $order_id = get_query_var( 'dm-order-id' ) ) {
			$this->manualOrder = true;
			$response          = $this->order_payment_complete( $order_id );

			$redirectUrl = get_query_var( 'dm-redirect' );
			wp_redirect( $redirectUrl ? $redirectUrl : get_edit_post_link( $order_id ) );
			exit;
		}
	}

	public function register_query_vars( $vars ) {
		$vars[] = 'dm-order-id';
		$vars[] = 'dm-redirect';

		return $vars;
	}

	/**
	 * Introduce Shipping Method for WC
	 *
	 * @param $methods
	 *
	 * @return mixed
	 */
	public function shipping_method( $methods ) {
		$methods['distrimedia_home']   = 'Distrimedia_Integration_Home';
		$methods['distrimedia_locker'] = 'Distrimedia_Integration_Locker';
		$methods['distrimedia_pickup'] = 'Distrimedia_Integration_Pickup';

		return $methods;
	}

	/**
	 * Injecting Extra Fields for bpost and google map
	 *
	 * @param $checkout
	 */
	public function extra_checkout_fields( $checkout ) {

		$points      = $this->bpost->get_bpost_points();
		$pointSelect = array();
		foreach ( $points as $point ) {
			$pointSelect[ $point->Id ] = $point->Name;
		}
		$googleApi = $this->bpost->get_google_api();
		require_once plugin_dir_path( __FILE__ ) . 'partials/checkout-fields.php';
	}


	/**
	 * Validate Extra Checkout fields here
	 */
	public function extra_checkout_fields_validation() {
		global $woocommerce;

		// Check if set, if its not set add an error.
		if (
			$this->bpost->has_shipping_method( $_POST['shipping_method'], 'distrimedia_locker' ) ||
			$this->bpost->has_shipping_method( $_POST['shipping_method'], 'distrimedia_pickup' )
//			in_array('distrimedia_locker', $_POST['shipping_method']) ||
//			in_array('distrimedia_pickup', $_POST['shipping_method'])
		) {

			$bpostPoint = false;

			if ( isset( $_POST['bpost_point'] ) ) {
				$bpostPoint = $this->bpost->get_bpost_station( sanitize_text_field($_POST['bpost_point']) );
			}

			if ( ! isset( $_POST['bpost_point'] ) || ! $_POST['bpost_point'] || ! $bpostPoint ) {
				wc_add_notice( '<strong>Please select the suitable bpost point</strong>', 'error' );
			} else {
				$_POST['ship_to_different_address'] = true;
				$_POST["shipping_first_name"]       = $bpostPoint->Name;
				$_POST["shipping_last_name"]        = ' #' . $bpostPoint->Id;
				$_POST["shipping_country"]          = 'BE';
				$_POST["shipping_address_1"]        = $bpostPoint->Street . ' ' . $bpostPoint->Number;
				$_POST["shipping_address_2"]        = '';
				$_POST["shipping_postcode"]         = $bpostPoint->Zip;
				$_POST["shipping_city"]             = $bpostPoint->City;
				$_POST["shipping_state"]            = '';
			}

		}

	}

	/**
	 * Update Order meta based on checkout fields
	 *
	 * @param $order_id
	 */
	public function extra_checkout_order_meta( $order_id ) {
		if ( $_POST['bpost_point'] ) {
			update_post_meta( $order_id, 'bpost_point', sanitize_text_field( $_POST['bpost_point'] ) );
		}
	}


}
